#import <AppKit/AppKit.h>

@interface Controller: NSObject
{
  NSWindow *window;
  NSTableView *tv;
}

- (int) numberOfRowsInTableView: (NSTableView *) view;
- (id) tableView: (NSTableView *) view
       objectValueForTableColumn: (NSTableColumn *) column
       row: (int) row;
- (BOOL) tableView: (NSTableView *) view
         writeRows: (NSArray *) rows
         toPasteboard: (NSPasteboard *) pboard;

@end

@implementation Controller;

- (BOOL) tableView: (NSTableView *) view
         writeRows: (NSArray *) rows
         toPasteboard: (NSPasteboard *) pboard
{
  NSLog(@"Dragged rows %@", rows);
  return NO;
}


- (int) numberOfRowsInTableView: (NSTableView *) view
{
  return 5;
}

- (id) tableView: (NSTableView *) view
       objectValueForTableColumn: (NSTableColumn *) column
       row: (int) row
{
  return [NSString stringWithFormat: @"Row %d", row];
}

- (BOOL) applicationShouldTerminateAfterLastWindowClosed: (id)sender
{
  return YES;
}

- (void) applicationDidFinishLaunching: (NSNotification *) not
{
  [window makeKeyAndOrderFront: self];
}

- (id) init
{
  NSTableColumn *tc;
  NSScrollView *sv;

  self = [super init];

  window = [[NSWindow alloc] initWithContentRect: NSMakeRect(200,200,400,300)
                                       styleMask: (NSTitledWindowMask |
                                                 NSClosableWindowMask |
                                           NSMiniaturizableWindowMask)
                                         backing: NSBackingStoreRetained
                                           defer: NO];

  tc = [[NSTableColumn alloc] initWithIdentifier: @"Column"];
  [[tc headerCell] setStringValue: @"Column"];

  sv = [[NSScrollView alloc] initWithFrame: [[window contentView] frame]];
  [sv setAutoresizingMask: (NSViewWidthSizable | NSViewHeightSizable)];

  tv = [[NSTableView alloc] initWithFrame: [[window contentView] frame]];
  [tv addTableColumn: tc];
  [tv setAutoresizesAllColumnsToFit: YES];
  [tv setAllowsMultipleSelection: YES];
  [tv setTarget: self];
  [tv setDataSource: self];
  [tv setDelegate: self];
  [tv registerForDraggedTypes: [NSArray arrayWithObjects: NSGeneralPboardType, nil]];

  [sv setDocumentView: tv];
  [[window contentView] addSubview: sv];

  [window setDelegate: self];

  return self;
}

@end

int main(int argc, const char **argv)
{
   NSAutoreleasePool *pool;
   Controller *controller;

   pool = [NSAutoreleasePool new];

   [NSApplication sharedApplication];

   controller = [[Controller alloc] init];

   [NSApp setDelegate: controller];

   NSApplicationMain(argc, argv);
	
   RELEASE(controller);
   RELEASE(pool);
   return 0;
}

